/*
EL.cpp
Written by Walter O. Krawec
Implementation of Evolution List STM system

Copyright (c) 2012 Walter O. Krawec

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#include "EL.h"
#include "ENN.h"

namespace MTENN
{
	float EvolutionList::maxFitness()
	{
		if(learningList.empty())
			return 0;

		float total = 0;
		std::list <LearningSet>::iterator Iter;
		for(Iter = learningList.begin(); Iter != learningList.end(); ++Iter)
		{
			if(Iter->r > 0)
				total += Iter->r;
		}

		return total;
	}
	float EvolutionList::minFitness()
	{
		if(learningList.empty())
			return 0;

		float total = 0;
		std::list <LearningSet>::iterator Iter;
		for(Iter = learningList.begin(); Iter != learningList.end(); ++Iter)
		{
			if(Iter->r < 0)
				total += Iter->r;
		}

		return total;
	}

	float EvolutionList::fitness(ENN* network)
	{
		if(!network)
			return 0;

		std::list <LearningSet>& evolutionList(getLearningList());

		if(evolutionList.empty())
			return 0.0f;

		float sum, total = 0.0f;
		std::list <LearningSet>::iterator Iter2;
		int avgIterations = 20;
		std::vector <LearningSet> v;
		for(Iter2 = evolutionList.begin(); Iter2 != evolutionList.end(); ++Iter2)
			v.push_back(*Iter2);
		avgIterations = 10;
		for(int m = 0; m<avgIterations; ++m)
		{
			std::vector <LearningSet>::iterator Iter;

			for(Iter = v.begin(); Iter != v.end(); ++Iter)
			{
				for(unsigned int i=0; i<Iter->inputs.size(); ++i)
					network->setInput(i, Iter->inputs[i]);

				network->run();

				sum = 0.0f;
				for(unsigned int i=0; i<Iter->outputs.size(); ++i)
				{
					float out = network->getOutput(i);
					//delta = 1 - (float)fabs(out - Iter->outputs[i]);
					//if(delta > 0.0f)
					//	sum += (Iter->r * delta);
					sum += ((out - Iter->outputs[i])*(out - Iter->outputs[i]));
				}

				sum = Iter->outputs.size()-sum;
				sum *= Iter->r;

				//sum = Iter->outputs.size() - sum;

				total += sum;
			}
		}

#ifdef MTENN_STM_DEBUG
		std::cout << total << "\n";
#endif
		total = total / (float)avgIterations;

		return total;
	}
}