/*
ULearn.cpp
Written by Walter O. Krawec
Implementation of U-Learning STM system

Copyright (c) 2012 Walter O. Krawec

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#include "ULearn.h"
#include "ENN.h"

namespace MTENN
{
	float ULearn::maxFitness()
	{
		float total = 0;
		for(int i=0; i<getStateSize(); ++i)
		{
			float m = U_Matrix.get(i,0);
			for(int j=1; j<getActionSize(); ++j)
			{
				if(U_Matrix.get(i,j) > m)
					m = U_Matrix.get(i,j);
			}

			total += m;
		}

		return total;
	}
	float ULearn::minFitness()
	{
		float total = 0;
		for(int i=0; i<getStateSize(); ++i)
		{
			float m = U_Matrix.get(i,0);
			for(int j=1; j<getActionSize(); ++j)
			{
				if(U_Matrix.get(i,j) < m)
					m = U_Matrix.get(i,j);
			}

			total += m;
		}

		return total;
	}

	float ULearn::fitness(ENN* network)
	{
		if(!network)
			return 0;

		float total = 0;
		int avgIterations = 10;
		unsigned int n = 1 << network->getNumInputs();
		std::vector <int> stateSpace;
		for(unsigned int i=0; i<n; ++i)
			stateSpace.push_back(i);

		for(int m=0; m<avgIterations; ++m)
		{
			float sum = 0;
			
			std::vector <float> outputs(network->getNumOutputs());

			std::random_shuffle(stateSpace.begin(), stateSpace.end());

			for(unsigned int i=0; i<n; ++i)
			{
#ifdef MTENN_STM_DEBUG
				std::cout << "\n" << i << ": (";
#endif

				int state = stateSpace[i];

				if(getRowSum(state) < 0.01f) continue;

				
				float localSum=0;
				int k = 1;
				for(unsigned int j=0; j<network->getNumInputs(); ++j)
				{
					network->setInput(j, (float)((state&k)/k));
#ifdef MTENN_STM_DEBUG
					std::cout << (float)((state&k)/k) << ", ";
#endif
					k = k << 1;
				}

#ifdef MTENN_STM_DEBUG
				std::cout << ") -> (";
#endif

				network->run();

				for(unsigned int j=0; j<network->getNumOutputs(); ++j)
				{
					outputs[j] = network->getOutput(j);
#ifdef MTENN_STM_DEBUG
					std::cout << outputs[j] << ", ";
#endif
				}

				float temp = getU(state, outputs);
#ifdef MTENN_STM_DEBUG
				std::cout << ") = " << temp;
#endif
				localSum += temp;
				
				sum += localSum;
			}
			total += sum;
		}

		total = total / (float)avgIterations;
		return total;
	}
}