/*
Tape.h
Written by Walter O. Krawec

Copyright (c) 2012 Walter O. Krawec

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef WOK_TAPE_H_
#define WOK_TAPE_H_

#include <vector>
#include <iostream>

namespace wok
{
	template <class T>
	class Tape
	{
	public:
		Tape() : MaxBufferSize(100), currentPosition(0) {};
		Tape(unsigned int maxBufferSize) : MaxBufferSize(maxBufferSize), currentPosition(0) {};
		Tape(const Tape<T>& t)
		{
			*this = t;
		};
		~Tape() {cleanup();}

		Tape<T>& operator= (const Tape& t)
		{
			if(this == &t)
				return *this;

			cleanup();
			MaxBufferSize = t.MaxBufferSize;
			T* buffer;
			for(unsigned int i=0; i<t.tapeList.size(); ++i)
			{
				buffer = new T[MaxBufferSize];
				for(unsigned int j=0; j<MaxBufferSize; ++j)
					buffer[j] = t.tapeList[i][j];
				tapeList.push_back(buffer);
			}

			currentPosition = t.currentPosition;

			return *this;
		}

		void cleanup()
		{
			for(unsigned int i=0; i<tapeList.size(); ++i)
			{
				if(tapeList[i])
				{
					//std::cout << "delete tape\n";
					delete[] tapeList[i];
				}
			}
			tapeList.clear();

			currentPosition = 0;
		};

		void clear()
		{
			if(tapeList.size() <= 1)
			{
				currentPosition = 0;
				return;
			}

			cleanup();
		}

		void append(Tape<T>& t)
		{
			for(unsigned int i=0; i<t.size(); ++i)
				push(t[i]);
		}

		void push(T data)
		{
			unsigned int vp = getVectorIndex(currentPosition);
			unsigned int bp = getBufferIndex(currentPosition);
			if(vp >= tapeList.size())
			{
				//std::cout << "new tape\n";
				tapeList.push_back(new T[MaxBufferSize]);
			}

			tapeList[vp][bp] = data;

			++currentPosition;
		}

		T operator[] (unsigned int index)
		{
			unsigned int vp = getVectorIndex(index);
			if(vp >= tapeList.size() || !tapeList[vp])
				return T();

			return tapeList[vp][getBufferIndex(index)];
		}

		T get(unsigned int index)
		{
			unsigned int vp = getVectorIndex(index);
			if(vp >= tapeList.size() || !tapeList[vp])
				return T();

			return tapeList[vp][getBufferIndex(index)];
		}

		unsigned int size() {return currentPosition;};


	private:
		unsigned int getVectorIndex(unsigned int position)
		{
			return position / MaxBufferSize;
		}
		unsigned int getBufferIndex(unsigned int position)
		{
			return (position % MaxBufferSize);
		}
		unsigned int MaxBufferSize;
		unsigned int currentPosition;
		std::vector < T* > tapeList;
	};
}

#endif