/*
Shapes.cpp
Written by Walter O. Krawec

Copyright (c) 2012 Walter O. Krawec

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#include "Shapes.h"

namespace SRS10{namespace graphics{
	
bool ShapeCollection::loadFromDataFile(const std::string& filename)
{
	std::ifstream f(filename.c_str());
	if(!f.is_open())
		return false;
	
	char buffer[1024];
	wok::TextParser text;
	
	text.addDelimiter('=');
	text.addDelimiter(',');
	text.addIgnore(' ');
	
	while(!f.eof())
	{
		f.getline(buffer, 1024);
		text.parse(buffer);
		
		if(text[0] == "EOF")
			break;
		else if(text[0] == "entity")
		{
			if(text[1] == "cluster")
			{
				// entity=cluster,centerX, centerY, radius, number, LT_min, LT_Max, RT_min, RT_Max, Value_min, Value_max
				float centerX = (float)atof(text[2].c_str());
				float centerY = (float)atof(text[3].c_str());
				float r = (float)atof(text[4].c_str());
				
				int number = atoi(text[5].c_str());
				int LT_min = atoi(text[6].c_str());
				int LT_max = atoi(text[7].c_str());
				int RT_min = atoi(text[8].c_str());
				int RT_max = atoi(text[9].c_str());
				int value_min = atoi(text[10].c_str());
				int value_max = atoi(text[11].c_str());
				
				for(int i=0; i<number; ++i)
				{
					SRS10::EntityManager::instance().add(SRS10::Entity( centerX + 2.0f*math3D::CMath3D::randFloat(r)-r,
																	   centerY + 2.0f*math3D::CMath3D::randFloat(r)-r,
																	   value_min + rand()%(value_max-value_min+1),
																	   LT_min+rand()%(LT_max-LT_min+1),
																	   RT_min+rand()%(RT_max-RT_min+1) ) );
				}
			}
		}
		else if(text[0] == "shape")
		{
			if(text[1] == "line")
			{
				Line* l = new Line;
				l->x1 = (float)atof(text[2].c_str());
				l->y1 = (float)atof(text[3].c_str());
				l->x2 = (float)atof(text[4].c_str());
				l->y2 = (float)atof(text[5].c_str());
				
				add(l);
			}
			else if(text[1] == "bold_line")
			{
				BoldLine *l = new BoldLine;
				l->x1 = (float)atof(text[2].c_str());
				l->y1 = (float)atof(text[3].c_str());
				l->x2 = (float)atof(text[4].c_str());
				l->y2 = (float)atof(text[5].c_str());

				l->solidObject = false;
				
				add(l);
			}
			else if(text[1] == "circle")
			{
				Circle* c = new Circle;
				c->x1 = (float)atof(text[2].c_str());
				c->y1 = (float)atof(text[3].c_str());
				c->radius = (float)atof(text[4].c_str());
				
				add(c);
			}
			else if(text[1] == "tri")
			{
				Triangle* t = new Triangle;
				t->x1 = (float)atof(text[2].c_str());
				t->y1 = (float)atof(text[3].c_str());
				t->baseSize = (float)atof(text[4].c_str());
				t->height = (float)atof(text[5].c_str());
				
				add(t);
			}
		}
	}
	
	f.close();
	
	return true;
}
}}