/*
* TextParser.h Written by Walter O. Krawec
* Started on May 14, 2006
* will take in a string and divide it into words
* seperated by the specified characters (default is
* by white space)
*/

/*
Copyright (c) 2012 Walter O. Krawec

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef WOK_TEXT_PARSER_H_
#define WOK_TEXT_PARSER_H_

#include <string>
#include <sstream>

namespace wok
{
	class TextParser
	{
	public:
		TextParser()
		{
			for(int i=0; i<10; ++i)
			{
				m_Delimiters[i] = 0;
				m_Ignore[i] = 0;
			}
		}

		TextParser(const std::string& str)
		{
			/*m_Delimiters.clear();
			m_Delimiters.add(' ');
			m_Delimiters.add('\t');*/
			for(int i=0; i<10; ++i)
			{
				m_Delimiters[i] = 0;
				m_Ignore[i] = 0;
			}

			parse(str);
		}
		//TextParser(const std::string& str, const wok::ArrayList<char>& delimiters) : m_Delimiters(10) {parse(str, delimiters);}
		~TextParser() {}

		void parse(const std::string& str)
		{
			m_ParseString = str;
			/*m_Delimiters.clear();
			m_Delimiters.add(' ');
			m_Delimiters.add('\t');*/

			m_LastStringCount = -1;
			m_LastString = "";
		}

		void clearDelimiterList()
		{
			for(int i=0; i<10; ++i)
				m_Delimiters[i] = 0;
		}
		void clearIgnoreList()
		{
			for(int i=0; i<10; ++i)
				m_Ignore[i] = 0;
		}
		void addDelimiter(char c)
		{
			for(int i=0; i<10; ++i)
			{
				if(m_Delimiters[i] == 0)
				{
					m_Delimiters[i] = c;
					break;
				}
			}
		}
		void addIgnore(char c)
		{
			for(int i=0; i<10; ++i)
			{
				if(m_Ignore[i] == 0)
				{
					m_Ignore[i] = c;
					break;
				}
			}
		}

		/*void parse(const std::string& str, const wok::ArrayList<char>& delimiters)
		{
			m_ParseString = str;
			m_Delimiters = delimiters;
			m_LastStringCount = -1;
			m_LastString = "";
		}*/

		// will return the number of words in m_ParseString
		unsigned int size()
		{
			unsigned int counter=0, position=0;
			do
			{
				// skip blank space:
				while(position < m_ParseString.length())
				{
					if(!checkDelimiters(m_ParseString[position]))
						break;	// we've found the next word

					++position;
				}

				if(position < m_ParseString.length())
					++counter;

				while(position < m_ParseString.length())
				{
					if(checkDelimiters(m_ParseString[position]))
						break;

					++position;
				}
			} while(position < m_ParseString.length());

			return counter;
		}

		std::string operator[] (unsigned int word)
		{
			if(m_ParseString.length() == 0)
				return m_ParseString;

			if(m_LastStringCount >= 0 && m_LastStringCount == (int)word)
				return m_LastString;

			unsigned int counter = 0;
			unsigned int position = 0;
			std::stringstream ret;
			do
			{
				// skip blank space:
				while(position < m_ParseString.length())
				{
					if(!checkDelimiters(m_ParseString[position]))
						break;	// we've found the next word

					++position;
				}
				if(counter == word)
				{
					// this is the word we want, so save it and return
					while(position < m_ParseString.length())
					{
						if(!checkIgnore(m_ParseString[position]))
							ret << m_ParseString[position];

						++position;
						if(position == m_ParseString.length())
							break;
						if(checkDelimiters(m_ParseString[position]))
							break;
					}

					m_LastString = ret.str();
					m_LastStringCount = word;
					return m_LastString;
				}

				// not the word we need, so skip it:
				while(position < m_ParseString.length())
				{
					if(checkDelimiters(m_ParseString[position]))
						break;

					++position;
				}

				++counter;
			} while(position < m_ParseString.length());

			m_LastString = ret.str();
			m_LastStringCount = word;
			return m_LastString;
		}

	private:
		bool checkDelimiters(char c)
		{
			for(int i=0; i<10; ++i)
			{
				if(m_Delimiters[i] != 0 && m_Delimiters[i] == c)
					return true;
			}

			return false;
		}
		bool checkIgnore(char c)
		{
			for(int i=0; i<10; ++i)
			{
				if(m_Ignore[i] != 0 && m_Ignore[i] == c)
					return true;
			}

			return false;
		}
		/*bool checkDelimiters(char c)
		{
			if(m_Delimiters.empty())
				return false;

			try {
				wok::ArrayList<char>::iterator Iter = m_Delimiters.begin();
				for(; !Iter.end(); ++Iter)
				{
					if(*Iter == c)
						return true;
				}
			} catch(std::string&) {}

			return false;
		}*/

		std::string m_ParseString, m_LastString;
		int m_LastStringCount;
		//wok::ArrayList<char> m_Delimiters;
		char m_Delimiters[10], m_Ignore[10];
	};
}

#endif