// Simple Robot Sim 2010 (SRS10)
// Copyright (c) 2012 Walter O. Krawec

/*
Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

// OpenGL init code contained in this file (main.cpp) thanks to Nehe at Gamedev.net

// A lot of this needs to be cleaned up and is probably more complicated than need be.  I started writing this simulator in 2010 (hence the name)
// and kept adding features as the ENN project "evolved".  This release is meant to be used for demonstrating the software concepts presented in
// the article.

// With some minor work it should work on a Mac (it worked there at one point) using GLUT

// This simulator is meant to work over a network.  When run, it will listen on port 50010 (which you can change; see "main()/WinMain()")
// Use SRS10Client.h to interface to the simulator over a network connection.  Multiple clients may connect (see Sockets.H; MAX_CLIENTS value)
// to a single simulator instance.

#include "Sockets\Sockets.h"

#ifdef WIN32
#include <windows.h>
#include <gl\gl.h>
#include <gl\glu.h>
#else
#include <OpenGL/gl.h>
#include <OpenGL/glu.h>
#include <OpenGL/glut.h>
#endif

#include "Track.h"
#include "Robots.h"
#include "Math3D.h"

// increase PACKET_VERSION whenever you modify the packet structure to avoid conflicts with older versions of srs10Client.h
#define PACKET_VERSION 2

SRS10::track::Track mainTrack;
SRS10::robot::RobotManager robotManager;

float cameraX = 0, cameraY = 0, cameraZ = -5;
bool simPaused = false;

wok::sockets::Host host;

#ifdef WIN32
HDC			hDC=NULL;		// Private GDI Device Context
HGLRC		hRC=NULL;		// Permanent Rendering Context
HWND		hWnd=NULL;		// Holds Our Window Handle
HINSTANCE	hInstance;		// Holds The Instance Of The Application
#endif

bool	keys[256];			// Array Used For The Keyboard Routine
bool	active=TRUE;		// Window Active Flag Set To TRUE By Default
bool	fullscreen=TRUE;	// Fullscreen Flag Set To Fullscreen Mode By Default

#ifdef WIN32
LRESULT	CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);	// Declaration For WndProc
#endif


Overlay hudANP;
GLvoid ReSizeGLScene(GLsizei width, GLsizei height)		// Resize And Initialize The GL Window
{
	if (height==0)										// Prevent A Divide By Zero By
	{
		height=1;										// Making Height Equal One
	}

	glViewport(0,0,width,height);						// Reset The Current Viewport

	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glLoadIdentity();									// Reset The Projection Matrix

	// Calculate The Aspect Ratio Of The Window
	gluPerspective(45.0f,(GLfloat)width/(GLfloat)height,0.1f,100.0f);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

int InitGL(GLvoid)										// All Setup For OpenGL Goes Here
{
	glShadeModel(GL_SMOOTH);							// Enable Smooth Shading
	glClearColor(0.0f, 0.0f, 0.0f, 0.5f);				// Black Background
	glClearDepth(1.0f);									// Depth Buffer Setup
	glEnable(GL_DEPTH_TEST);							// Enables Depth Testing
	glDepthFunc(GL_LEQUAL);								// The Type Of Depth Testing To Do
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);	// Really Nice Perspective Calculations
	return TRUE;										// Initialization Went OK
}

bool verifyVersion(wok::sockets::Packet& p)
{
	if(p.unpackChar() != PACKET_VERSION)
		return false;
	return true;
}

void sendError(wok::sockets::Socket* socket, int message)
{
	if(!socket)
		return;
	
	wok::sockets::Packet p;
	p.createHeader(3);
	p.packChar(message);
	p.packChar(PACKET_VERSION);
	socket->sendPacket(p);
}

void hostLoop()
{
	wok::sockets::Packet p;
	wok::sockets::Socket* socket;
	host.mainLoop();
	for(int clientNum=0; clientNum < wok::sockets::MAX_CLIENTS; ++clientNum)
	{
		socket = host.getClientSocket(clientNum);
		if(!socket)
			continue;
		int packetType = socket->grabNextPacket(p);
		if(packetType != -1 && !verifyVersion(p))
			continue;
		if(packetType == 1)
		{
			char type = p.unpackChar();
			SRS10::robot::Robot* robot1 = new SRS10::robot::Robot;
			robot1->type = type;
			robot1->loadFromFile("data/Robot/Robot1.txt");
			robot1->setRotation(rand()%360);

			math3D::CVector2D cp;
			mainTrack.getCreationPoint(cp, type);

			robot1->setPosition(cp.x + math3D::CMath3D::randFloat(2)-1, cp.y+math3D::CMath3D::randFloat(2)-1);
			robot1->setRotation(math3D::CMath3D::randFloat(360));
			int ID = robotManager.add(robot1);
			
			p.createHeader(1);
			p.packInt(ID);
			p.packChar(PACKET_VERSION);
			socket->sendPacket(p);
		}
		else if(packetType == 2)
		{
			int robotID = p.unpackInt();
			SRS10::robot::Robot* robot = robotManager.getRobot(robotID);
			if(!robot)
			{
				sendError(socket, 1);
				continue;
			}
			int type = p.unpackChar();
			if(type == 1 || type == 2)
			{
				float range = p.unpackFloat();
				float rot = p.unpackFloat();
				wok::UID ID;
				if(type == 1)
					ID = robot->getSensorManager().add(new SRS10::robot::IR(0,0,rot,range));
				else
					ID = robot->getSensorManager().add(new SRS10::robot::IR360(0,0,rot,range, p.unpackChar()));
				
				p.createHeader(2);
				p.packInt(ID.get());
				p.packChar(PACKET_VERSION);
				socket->sendPacket(p);
			}
			else if(type == 3)
			{
				float y = p.unpackFloat();
				float x = p.unpackFloat();

				wok::UID ID;
				ID = robot->getSensorManager().add(new SRS10::robot::LightSensor(x, y));

				p.createHeader(2);
				p.packInt(ID.get());
				p.packChar(PACKET_VERSION);
				socket->sendPacket(p);
			}
			else if(type == 4)
			{
				int quad = p.unpackInt();
				float range = p.unpackFloat();

				wok::UID ID;
				ID = robot->getSensorManager().add(new SRS10::robot::ProximitySensor(range, quad));

				p.createHeader(2);
				p.packInt(ID.get());
				p.packChar(PACKET_VERSION);
				socket->sendPacket(p);
			}
			else if(type == 5)
			{
				float y = p.unpackFloat();
				float x = p.unpackFloat();

				wok::UID ID;
				ID = robot->getSensorManager().add(new SRS10::robot::LineSensor(x, y));

				p.createHeader(2);
				p.packInt(ID.get());
				p.packChar(PACKET_VERSION);
				socket->sendPacket(p);
			}
		}
		else if(packetType == 20)
		{
			SRS10::robot::Robot* robot = robotManager.getRobot(p.unpackInt());
			if(!robot)
			{
				sendError(socket, 1);
				continue;
			}
			
			std::string message;
			p.unpackString(message);
			SRS10::graphics::GraphicsEngine::instance().addMessage(message, robot->getX(), robot->getY()+0.2f, false);
		}
		else if(packetType == 10)
		{
			SRS10::robot::Robot* robot = robotManager.getRobot(p.unpackInt());
			if(!robot)
			{
				sendError(socket, 1);
				continue;
			}

			int special = p.unpackChar();
			
			robot->runMoveCommand(p.unpackChar(), special);
		}
		else if(packetType == 11)
		{
			int callID = p.unpackInt();
			SRS10::robot::Robot* robot = robotManager.getRobot(p.unpackInt());
			if(!robot)
			{
				sendError(socket, 1);
				continue;
			}
			
			p.createHeader(11);
			robot->packSensorValues(p, mainTrack, robotManager);
			p.packInt(callID);
			p.packChar(PACKET_VERSION);
			socket->sendPacket(p);
		}
		else if(packetType == 21)
		{
			int callID = p.unpackInt();
			SRS10::robot::Robot* robot = robotManager.getRobot(p.unpackInt());
			if(!robot)
			{
				sendError(socket, 1);
				continue;
			}
			
			p.createHeader(21);
			robot->packEvents(p, mainTrack, robotManager);
			p.packInt(callID);
			p.packChar(PACKET_VERSION);
			socket->sendPacket(p);
			
			robot->clearEvents();
		}
		else if(packetType == 12)
		{
			int robotID = p.unpackInt();
			int sensorID = p.unpackInt();
			
			SRS10::robot::Robot* robot = robotManager.getRobot(robotID);
			if(!robot)
			{
				sendError(socket, 1);
				continue;
			}
			
			SRS10::robot::Sensor* sensor = robot->getSensorManager().get(sensorID);
			if(!sensor)
			{
				sendError(socket, 2);
				continue;
			}
			
			float arg2 = p.unpackFloat();
			float arg1 = p.unpackFloat();
			sensor->set(arg1, arg2);
		}
		else if(packetType == 13)
		{
			int robotID = p.unpackInt();
			SRS10::robot::Robot* robot = robotManager.getRobot(robotID);
			if(!robot)
			{
				sendError(socket, 1);
				continue;
			}

			robot->outputVector.clear();
			int N = p.unpackInt();
			for(int i=0; i<N; ++i)
				robot->outputVector.push_back( (int)p.unpackChar() );

			robot->inputVector.clear();
			N = p.unpackInt();
			for(int i=0; i<N; ++i)
				robot->inputVector.push_back( (int)p.unpackChar() );
		}
		else if(packetType == 14)
		{
			int robotID = p.unpackInt();
			SRS10::robot::Robot* robot = robotManager.getRobot(robotID);
			if(!robot)
			{
				sendError(socket, 1);
				continue;
			}

			char creator = p.unpackChar();
			float volume = p.unpackFloat();
			int r = p.unpackInt();

			float blue = p.unpackFloat();
			float green = p.unpackFloat();
			float red = p.unpackFloat();

			mainTrack.addFX(SRS10::track::VisualFX(robot->x,robot->y, volume, red, green, blue,500));
			robotManager.addGlobalEvent(robot->x,robot->y,r,creator, volume, robotID);
		}
		else if(packetType == 50)
		{
			int robotID = p.unpackInt();
			SRS10::robot::Robot* robot = robotManager.getRobot(robotID);
			if(!robot)
			{
				sendError(socket, 1);
				continue;
			}

			robot->useANP = true;
		}
		else if(packetType == 51)
		{
			int robotID = p.unpackInt();
			SRS10::robot::Robot* robot = robotManager.getRobot(robotID);
			if(!robot)
			{
				sendError(socket, 1);
				continue;
			}

			robot->startANP();
		}
		else if(packetType == 52)
		{
			int robotID = p.unpackInt();
			SRS10::robot::Robot* robot = robotManager.getRobot(robotID);
			if(!robot)
			{
				sendError(socket, 1);
				continue;
			}

			if(robot->anp.isScanning())
			{
				p.createHeader(52);
				p.packChar(0);
			}
			else
			{
				p.createHeader(52);
				for(int i=0; i<robot->anp.getMaxNavPoints(); ++i)
					p.packInt(robot->anp.getNavPoint(robot->anp.getMaxNavPoints()-i-1));

				p.packInt(robot->anp.getMaxNavPoints());
				p.packChar(1);
			}

			p.packChar(PACKET_VERSION);
			socket->sendPacket(p);
		}
		else if(packetType == 53)
		{
			int robotID = p.unpackInt();
			SRS10::robot::Robot* robot = robotManager.getRobot(robotID);
			if(!robot)
			{
				sendError(socket, 1);
				continue;
			}

			robot->anpNavToPoint(p.unpackInt());
		}
		else if(packetType == 54)
		{
			int robotID = p.unpackInt();
			SRS10::robot::Robot* robot = robotManager.getRobot(robotID);
			if(!robot)
			{
				sendError(socket, 1);
				continue;
			}

			char collision = 0;
			char ready = 1;
			if(robot->anp.collision())
				collision = 1;

			if(robot->anp.isMoving() || robot->anp.isScanning())
				ready = 0;

			p.createHeader(54);
			p.packChar(collision);
			p.packChar(ready);
			p.packChar(PACKET_VERSION);
			socket->sendPacket(p);
		}
	}
}

bool mouseLButtonDown = false;
POINT mouseLButtonPos;

void moveRobot()
{
	POINT p;
	GetCursorPos(&p);

	float mouseX = ((float)p.x - 320.0f)/320.0f;
	float mouseY = ((float)p.y - 240.0f)/240.0f;

	float scaleX = 2.5f - 0.5f*(cameraZ+5);
	float scaleY = 2 - 0.5f*(cameraZ+5);
	float x = (mouseX*scaleX - cameraX);
	float y = (-mouseY*scaleY - cameraY);

	for(unsigned int i=0; i<robotManager.getRobotList().size(); ++i)
	{
		SRS10::robot::Robot* r = robotManager.getRobotList()[i];
		if(!r)
			return;
		r->setPosition(x,y);
		break;
	}
}
void mouseRButtonClick()
{
	POINT p;
	GetCursorPos(&p);

	float mouseX = ((float)p.x - 320.0f)/320.0f;
	float mouseY = ((float)p.y - 240.0f)/240.0f;

	float scaleX = 2.5f - 0.5f*(cameraZ+5);
	float scaleY = 2 - 0.5f*(cameraZ+5);
	float x = (mouseX*scaleX - cameraX);
	float y = (-mouseY*scaleY - cameraY);
	float volume = 4.0f;

	mainTrack.addFX(SRS10::track::VisualFX(x,y, 0.25f, 1,1,1,500));
	robotManager.addGlobalEvent(x,y,0,0, volume);
}

int DrawGLScene(GLvoid)									// Here's Where We Do All The Drawing
{
	if(keys[VK_F1] && keys[VK_F12])
	{
		keys[VK_F1] = false;
		keys[VK_F12] = false;

		robotManager.cleanup();
	}

	if(mouseLButtonDown)
	{
		POINT p;
		GetCursorPos(&p);

		float scaleX = 0.01f;
		float scaleY = 0.01f;

		cameraX -= (scaleX * (mouseLButtonPos.x - p.x));
		cameraY += (scaleY * (mouseLButtonPos.y - p.y));

		mouseLButtonPos = p;
	}

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// Clear Screen And Depth Buffer
	glLoadIdentity();									// Reset The Current Modelview Matrix

	SRS10::graphics::GraphicsEngine::instance().displayMessages(-2.5f,1.8f);
	glTranslatef(cameraX, cameraY, cameraZ);
	
	SRS10::EntityManager::instance().draw();
	
	mainTrack.draw();
	
	if(!simPaused)
		robotManager.updateAll(mainTrack);
	else
		SRS10::graphics::GraphicsEngine::instance().printString("Press SPACE to run simulation",0,0,0);

	hudANP.draw();
	
 	SRS10::graphics::GraphicsEngine::instance().updateFPS();
	
	std::stringstream str;
	str << "FPS = " << SRS10::graphics::GraphicsEngine::instance().getFPS();
//	SRS10::graphics::GraphicsEngine::instance().printString(str.str(), 0,0);
	
	SRS10::graphics::GraphicsEngine::instance().displayFloatingMessages();

	hostLoop();

#ifndef WIN32
	glutSwapBuffers();
#endif
	return TRUE;										// Keep Going
}

#ifndef WIN32
void idleLoop()
{
	glutPostRedisplay();
}

void processSpecialKeys(int key, int x, int y)
{
}

void processNormalKeys(unsigned char key, int x, int y)
{
	keys[key] = true;
}
#endif

#ifdef WIN32
GLvoid KillGLWindow(GLvoid)								// Properly Kill The Window
{
	if (fullscreen)										// Are We In Fullscreen Mode?
	{
		ChangeDisplaySettings(NULL,0);					// If So Switch Back To The Desktop
		ShowCursor(TRUE);								// Show Mouse Pointer
	}

	if (hRC)											// Do We Have A Rendering Context?
	{
		if (!wglMakeCurrent(NULL,NULL))					// Are We Able To Release The DC And RC Contexts?
		{
			MessageBox(NULL,"Release Of DC And RC Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		}

		if (!wglDeleteContext(hRC))						// Are We Able To Delete The RC?
		{
			MessageBox(NULL,"Release Rendering Context Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		}
		hRC=NULL;										// Set RC To NULL
	}

	if (hDC && !ReleaseDC(hWnd,hDC))					// Are We Able To Release The DC
	{
		MessageBox(NULL,"Release Device Context Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hDC=NULL;										// Set DC To NULL
	}

	if (hWnd && !DestroyWindow(hWnd))					// Are We Able To Destroy The Window?
	{
		MessageBox(NULL,"Could Not Release hWnd.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hWnd=NULL;										// Set hWnd To NULL
	}

	if (!UnregisterClass("OpenGL",hInstance))			// Are We Able To Unregister Class
	{
		MessageBox(NULL,"Could Not Unregister Class.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hInstance=NULL;									// Set hInstance To NULL
	}
}

/*	This Code Creates Our OpenGL Window.  Parameters Are:					*
 *	title			- Title To Appear At The Top Of The Window				*
 *	width			- Width Of The GL Window Or Fullscreen Mode				*
 *	height			- Height Of The GL Window Or Fullscreen Mode			*
 *	bits			- Number Of Bits To Use For Color (8/16/24/32)			*
 *	fullscreenflag	- Use Fullscreen Mode (TRUE) Or Windowed Mode (FALSE)	*/
 
BOOL CreateGLWindow(char* title, int width, int height, int bits, bool fullscreenflag)
{
	GLuint		PixelFormat;			// Holds The Results After Searching For A Match
	WNDCLASS	wc;						// Windows Class Structure
	DWORD		dwExStyle;				// Window Extended Style
	DWORD		dwStyle;				// Window Style
	RECT		WindowRect;				// Grabs Rectangle Upper Left / Lower Right Values
	WindowRect.left=(long)0;			// Set Left Value To 0
	WindowRect.right=(long)width;		// Set Right Value To Requested Width
	WindowRect.top=(long)0;				// Set Top Value To 0
	WindowRect.bottom=(long)height;		// Set Bottom Value To Requested Height

	fullscreen=fullscreenflag;			// Set The Global Fullscreen Flag

	hInstance			= GetModuleHandle(NULL);				// Grab An Instance For Our Window
	wc.style			= CS_HREDRAW | CS_VREDRAW | CS_OWNDC;	// Redraw On Size, And Own DC For Window.
	wc.lpfnWndProc		= (WNDPROC) WndProc;					// WndProc Handles Messages
	wc.cbClsExtra		= 0;									// No Extra Window Data
	wc.cbWndExtra		= 0;									// No Extra Window Data
	wc.hInstance		= hInstance;							// Set The Instance
	wc.hIcon			= LoadIcon(NULL, IDI_WINLOGO);			// Load The Default Icon
	wc.hCursor			= LoadCursor(NULL, IDC_ARROW);			// Load The Arrow Pointer
	wc.hbrBackground	= NULL;									// No Background Required For GL
	wc.lpszMenuName		= NULL;									// We Don't Want A Menu
	wc.lpszClassName	= "OpenGL";								// Set The Class Name

	if (!RegisterClass(&wc))									// Attempt To Register The Window Class
	{
		MessageBox(NULL,"Failed To Register The Window Class.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;											// Return FALSE
	}
	
	if (fullscreen)												// Attempt Fullscreen Mode?
	{
		DEVMODE dmScreenSettings;								// Device Mode
		memset(&dmScreenSettings,0,sizeof(dmScreenSettings));	// Makes Sure Memory's Cleared
		dmScreenSettings.dmSize=sizeof(dmScreenSettings);		// Size Of The Devmode Structure
		dmScreenSettings.dmPelsWidth	= width;				// Selected Screen Width
		dmScreenSettings.dmPelsHeight	= height;				// Selected Screen Height
		dmScreenSettings.dmBitsPerPel	= bits;					// Selected Bits Per Pixel
		dmScreenSettings.dmFields=DM_BITSPERPEL|DM_PELSWIDTH|DM_PELSHEIGHT;

		// Try To Set Selected Mode And Get Results.  NOTE: CDS_FULLSCREEN Gets Rid Of Start Bar.
		if (ChangeDisplaySettings(&dmScreenSettings,CDS_FULLSCREEN)!=DISP_CHANGE_SUCCESSFUL)
		{
			// If The Mode Fails, Offer Two Options.  Quit Or Use Windowed Mode.
			if (MessageBox(NULL,"The Requested Fullscreen Mode Is Not Supported By\nYour Video Card. Use Windowed Mode Instead?","NeHe GL",MB_YESNO|MB_ICONEXCLAMATION)==IDYES)
			{
				fullscreen=FALSE;		// Windowed Mode Selected.  Fullscreen = FALSE
			}
			else
			{
				// Pop Up A Message Box Letting User Know The Program Is Closing.
				MessageBox(NULL,"Program Will Now Close.","ERROR",MB_OK|MB_ICONSTOP);
				return FALSE;									// Return FALSE
			}
		}
	}

	if (fullscreen)												// Are We Still In Fullscreen Mode?
	{
		dwExStyle=WS_EX_APPWINDOW;								// Window Extended Style
		dwStyle=WS_POPUP;										// Windows Style
		ShowCursor(FALSE);										// Hide Mouse Pointer
	}
	else
	{
		dwExStyle=WS_EX_APPWINDOW | WS_EX_WINDOWEDGE;			// Window Extended Style
		dwStyle=WS_OVERLAPPEDWINDOW;							// Windows Style
	}

	AdjustWindowRectEx(&WindowRect, dwStyle, FALSE, dwExStyle);		// Adjust Window To True Requested Size

	// Create The Window
	if (!(hWnd=CreateWindowEx(	dwExStyle,							// Extended Style For The Window
								"OpenGL",							// Class Name
								title,								// Window Title
								dwStyle |							// Defined Window Style
								WS_CLIPSIBLINGS |					// Required Window Style
								WS_CLIPCHILDREN,					// Required Window Style
								0, 0,								// Window Position
								WindowRect.right-WindowRect.left,	// Calculate Window Width
								WindowRect.bottom-WindowRect.top,	// Calculate Window Height
								NULL,								// No Parent Window
								NULL,								// No Menu
								hInstance,							// Instance
								NULL)))								// Dont Pass Anything To WM_CREATE
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Window Creation Error.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	static	PIXELFORMATDESCRIPTOR pfd=				// pfd Tells Windows How We Want Things To Be
	{
		sizeof(PIXELFORMATDESCRIPTOR),				// Size Of This Pixel Format Descriptor
		1,											// Version Number
		PFD_DRAW_TO_WINDOW |						// Format Must Support Window
		PFD_SUPPORT_OPENGL |						// Format Must Support OpenGL
		PFD_DOUBLEBUFFER,							// Must Support Double Buffering
		PFD_TYPE_RGBA,								// Request An RGBA Format
		bits,										// Select Our Color Depth
		0, 0, 0, 0, 0, 0,							// Color Bits Ignored
		0,											// No Alpha Buffer
		0,											// Shift Bit Ignored
		0,											// No Accumulation Buffer
		0, 0, 0, 0,									// Accumulation Bits Ignored
		16,											// 16Bit Z-Buffer (Depth Buffer)  
		0,											// No Stencil Buffer
		0,											// No Auxiliary Buffer
		PFD_MAIN_PLANE,								// Main Drawing Layer
		0,											// Reserved
		0, 0, 0										// Layer Masks Ignored
	};
	
	if (!(hDC=GetDC(hWnd)))							// Did We Get A Device Context?
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Create A GL Device Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	if (!(PixelFormat=ChoosePixelFormat(hDC,&pfd)))	// Did Windows Find A Matching Pixel Format?
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Find A Suitable PixelFormat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	if(!SetPixelFormat(hDC,PixelFormat,&pfd))		// Are We Able To Set The Pixel Format?
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Set The PixelFormat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	if (!(hRC=wglCreateContext(hDC)))				// Are We Able To Get A Rendering Context?
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Create A GL Rendering Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	if(!wglMakeCurrent(hDC,hRC))					// Try To Activate The Rendering Context
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Activate The GL Rendering Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	ShowWindow(hWnd,SW_SHOW);						// Show The Window
	SetForegroundWindow(hWnd);						// Slightly Higher Priority
	SetFocus(hWnd);									// Sets Keyboard Focus To The Window
	ReSizeGLScene(width, height);					// Set Up Our Perspective GL Screen

	if (!InitGL())									// Initialize Our Newly Created GL Window
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Initialization Failed.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	return TRUE;									// Success
}

LRESULT CALLBACK WndProc(	HWND	hWnd,			// Handle For This Window
							UINT	uMsg,			// Message For This Window
							WPARAM	wParam,			// Additional Message Information
							LPARAM	lParam)			// Additional Message Information
{
	switch (uMsg)									// Check For Windows Messages
	{
		case WM_ACTIVATE:							// Watch For Window Activate Message
		{
			if (!HIWORD(wParam))					// Check Minimization State
			{
				active=TRUE;						// Program Is Active
			}
			else
			{
				active=FALSE;						// Program Is No Longer Active
			}

			return 0;								// Return To The Message Loop
		}

		case WM_SYSCOMMAND:
		{
			switch (wParam)
			{
				case SC_SCREENSAVE:
				case SC_MONITORPOWER:
					return 0;
			}
			break;
		}

		case WM_CLOSE:								// Did We Receive A Close Message?
		{
			PostQuitMessage(0);						// Send A Quit Message
			return 0;								// Jump Back
		}
		case WM_MOUSEWHEEL:
		{
			const float ZOOM_SPEED = 0.1f;
			int dir = wParam;
			if(dir >= 0)
				cameraZ += ZOOM_SPEED;
			else
				cameraZ -= ZOOM_SPEED;
			break;
		}

		case WM_RBUTTONDOWN:
			mouseRButtonClick();
			break;
		case WM_LBUTTONDOWN:
			if(keys[VK_SHIFT])
				moveRobot();
			else
			{
			mouseLButtonDown = true;
			GetCursorPos(&mouseLButtonPos);
			}
			break;

		case WM_LBUTTONDBLCLK:
			moveRobot();
			break;

		case WM_LBUTTONUP:
			mouseLButtonDown = false;
			break;

		case WM_KEYDOWN:							// Is A Key Being Held Down?
		{
			keys[wParam] = TRUE;					// If So, Mark It As TRUE
			if(wParam == 'C')
				robotManager.addGlobalEvent(0, 0, 1, 0, -1);
			else if(wParam == 'I')
				robotManager.addGlobalEvent(0, 0, -1, 0, -1);
			return 0;								// Jump Back
		}

		case WM_KEYUP:								// Has A Key Been Released?
		{
			keys[wParam] = FALSE;					// If So, Mark It As FALSE
			return 0;								// Jump Back
		}

		case WM_SIZE:								// Resize The OpenGL Window
		{
			ReSizeGLScene(LOWORD(lParam),HIWORD(lParam));  // LoWord=Width, HiWord=Height
			return 0;								// Jump Back
		}
	}

	// Pass All Unhandled Messages To DefWindowProc
	return DefWindowProc(hWnd,uMsg,wParam,lParam);
}
#endif

bool startHost(int portNumber)
{
	host.getMainSocket()->initWinSock();
	return host.setupHost(portNumber);
}

#ifdef WIN32
int WINAPI WinMain(	HINSTANCE	hInstance,			// Instance
					HINSTANCE	hPrevInstance,		// Previous Instance
					LPSTR		lpCmdLine,			// Command Line Parameters
					int			nCmdShow)			// Window Show State
{
	MSG		msg;									// Windows Message Structure
	BOOL	done=FALSE;								// Bool Variable To Exit Loop

	// Ask The User Which Screen Mode They Prefer
	//if (MessageBox(NULL,"Would You Like To Run In Fullscreen Mode?", "Start FullScreen?",MB_YESNO|MB_ICONQUESTION)==IDNO)
	{
		fullscreen=FALSE;							// Windowed Mode
	}

	//hudANP.addSquare(Overlay::Point());

	srand(time(NULL));
	if(!startHost(50010))
		return 0;

	// Create Our OpenGL Window
	if (!CreateGLWindow("Simple Robot Sim 2010",640,480,16,fullscreen))
	{
		return 0;									// Quit If Window Was Not Created
	}

	if(!mainTrack.loadFromFile("Data/Track/track1.txt"))
	{
		KillGLWindow();
		return 0;
	}

	while(!done)									// Loop That Runs While done=FALSE
	{
		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))	// Is There A Message Waiting?
		{
			if (msg.message==WM_QUIT)				// Have We Received A Quit Message?
			{
				done=TRUE;							// If So done=TRUE
			}
			else									// If Not, Deal With Window Messages
			{
				TranslateMessage(&msg);				// Translate The Message
				DispatchMessage(&msg);				// Dispatch The Message
			}
		}
		else										// If There Are No Messages
		{
			// Draw The Scene.  Watch For ESC Key And Quit Messages From DrawGLScene()
			if ((active && !DrawGLScene()) || keys[VK_ESCAPE])	// Active?  Was There A Quit Received?
			{
				done=TRUE;							// ESC or DrawGLScene Signalled A Quit
			}
			else									// Not Time To Quit, Update Screen
			{
				SwapBuffers(hDC);					// Swap Buffers (Double Buffering)
			}

			/*if (keys[VK_F1])						// Is F1 Being Pressed?
			{
				keys[VK_F1]=FALSE;					// If So Make Key FALSE
				KillGLWindow();						// Kill Our Current Window
				fullscreen=!fullscreen;				// Toggle Fullscreen / Windowed Mode
				// Recreate Our OpenGL Window
				if (!CreateGLWindow("NeHe's First Polygon Tutorial",640,480,16,fullscreen))
				{
					return 0;						// Quit If Window Was Not Created
				}
			}*/
		}
	}

	// Shutdown
	KillGLWindow();									// Kill The Window
	return (msg.wParam);							// Exit The Program
}
#else
int main(int argc, char** argv)
{
	srand(time(NULL));

	if(!startHost(50010))
		return 0;

	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowSize(640, 480);
	glutInitWindowPosition(100,100);
	glutCreateWindow(argv[0]);

	InitGL();

	if(!mainTrack.loadFromFile("../../Data/Track/track1.txt"))
		return 0;

	glutDisplayFunc(DrawGLScene);
	glutReshapeFunc(ReSizeGLScene);
	glutKeyboardFunc(processNormalKeys);
	glutSpecialFunc(processSpecialKeys);

	glutIdleFunc(idleLoop);
	glutMainLoop();
	return 0;
}
#endif